"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Default = exports.IndexDefaultMethod = exports.getArrayProperty = exports.IdentifyArrayProperty = exports.TypeOfArray = void 0;
require("reflect-metadata");
const CUSTOM_METADATA = Symbol('CustomMetadata');
var TypeOfArray;
(function (TypeOfArray) {
    TypeOfArray[TypeOfArray["STRING_ARRAY"] = 0] = "STRING_ARRAY";
    TypeOfArray[TypeOfArray["OBJECT_ARRAY"] = 1] = "OBJECT_ARRAY";
})(TypeOfArray = exports.TypeOfArray || (exports.TypeOfArray = {}));
const IdentifyArrayProperty = (type) => Reflect.metadata(CUSTOM_METADATA, type);
exports.IdentifyArrayProperty = IdentifyArrayProperty;
const getArrayProperty = (obj, key) => Reflect.getMetadata(CUSTOM_METADATA, obj, key);
exports.getArrayProperty = getArrayProperty;
const defaultMethods = new Map();
const IndexDefaultMethod = () => {
    return (target, memberName, propertyDescriptor) => {
        defaultMethods.set(target.name.toLowerCase(), target[memberName]);
    };
};
exports.IndexDefaultMethod = IndexDefaultMethod;
class Default {
    assign(source) {
        let key;
        const keys = Object.keys(source);
        for (key of keys) {
            let innerItem;
            const typeOfArray = exports.getArrayProperty(this, key);
            if (typeOfArray === TypeOfArray.STRING_ARRAY) {
                const tab = source[key] instanceof Array ? source[key] : [source[key]];
                const tmp = [];
                tab.forEach((add) => {
                    tmp.push(add);
                });
                innerItem = tmp;
            }
            else if (typeOfArray === TypeOfArray.OBJECT_ARRAY) {
                const currentDefaultMethod = defaultMethods.get(key.toLowerCase());
                if (currentDefaultMethod) {
                    const tab = source[key] instanceof Array ? source[key] : [source[key]];
                    const tmp = [];
                    tab.forEach((add) => {
                        tmp.push(currentDefaultMethod().assign(add));
                    });
                    innerItem = tmp;
                }
            }
            else if (source[key] instanceof Object) {
                const currentDefaultMethod = defaultMethods.get(key.toLowerCase());
                if (currentDefaultMethod)
                    innerItem = currentDefaultMethod().assign(source[key]);
            }
            else
                innerItem = isNumber(source[key]) ? Number(source[key]) : source[key];
            if (innerItem)
                Object.defineProperty(this, key, { value: innerItem, enumerable: true, writable: true });
        }
        return this;
    }
    static default() {
        throw new Error('Default method not implemented !');
    }
}
exports.Default = Default;
const isNumber = (n) => !isNaN(parseFloat(String(n))) && isFinite(Number(n));
