"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Mission = void 0;
const uuid_1 = require("uuid");
const config_1 = require("../../common/config");
const default_1 = require("../../common/default");
const types_1 = require("../../common/types");
const error_1 = require("../../error");
class Mission extends default_1.Default {
    constructor(type, id) {
        super();
        error_1.MissionError.checkLength(id, config_1.MAX_ID_LENGTH);
        this.type = type;
        this.id = id || uuid_1.v4();
    }
    setFreeText(freeText) {
        error_1.MissionError.checkLength(freeText, config_1.MAX_FREETEXT_LENGTH);
        this.freeText = freeText;
        return this;
    }
    setMainMissionId(mainId) {
        error_1.MissionError.checkLength(mainId, config_1.MAX_ID_LENGTH);
        this.mainMissionId = mainId;
        return this;
    }
    setOrgId(orgId) {
        error_1.MissionError.checkLength(orgId, config_1.MAX_ID_LENGTH);
        this.orgId = orgId;
        return this;
    }
    setName(name) {
        error_1.MissionError.checkLength(name, config_1.MAX_NAME_X2_LENGTH);
        this.name = name;
        return this;
    }
    setStatus(status, completeness) {
        if (status === types_1.MissionStatus.IN_PROGRESS && completeness)
            if (completeness > 0 && completeness < 100)
                this.status = types_1.MissionStatus.IN_PROGRESS + completeness.toString().padStart(2, '0');
            else
                throw new error_1.MissionError('Percentage of completeness of the mission should be between 1 and 99');
        else
            this.status = status;
        return this;
    }
    setStartTime(start) {
        this.startTime = start.toISOString();
        return this;
    }
    setEndTime(end) {
        this.endTime = end.toISOString();
        return this;
    }
    addResourceIds(resourceIds) {
        resourceIds.forEach(add => error_1.MissionError.checkLength(add, config_1.MAX_RESOURCE_ID_LENGTH));
        if (!this.resourceId)
            this.resourceId = new Array();
        this.resourceId.push(...resourceIds);
        return this;
    }
    addParentMissionIds(parentIds) {
        parentIds.forEach(add => error_1.MissionError.checkLength(add, config_1.MAX_ID_LENGTH));
        if (!this.parentMissionId)
            this.parentMissionId = new Array();
        this.parentMissionId.push(...parentIds);
        return this;
    }
    addChildMissionIds(childIds) {
        childIds.forEach(add => error_1.MissionError.checkLength(add, config_1.MAX_ID_LENGTH));
        if (!this.childMissionId)
            this.childMissionId = new Array();
        this.childMissionId.push(...childIds);
        return this;
    }
    setPosition(position) {
        this.position = position;
        return this;
    }
    setPriority(priority) {
        this.priority = priority;
        return this;
    }
    static default() {
        return new Mission('');
    }
}
__decorate([
    default_1.IdentifyArrayProperty(default_1.TypeOfArray.STRING_ARRAY),
    __metadata("design:type", Array)
], Mission.prototype, "resourceId", void 0);
__decorate([
    default_1.IdentifyArrayProperty(default_1.TypeOfArray.STRING_ARRAY),
    __metadata("design:type", Array)
], Mission.prototype, "parentMissionId", void 0);
__decorate([
    default_1.IdentifyArrayProperty(default_1.TypeOfArray.STRING_ARRAY),
    __metadata("design:type", Array)
], Mission.prototype, "childMissionId", void 0);
__decorate([
    default_1.IndexDefaultMethod(),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", Mission)
], Mission, "default", null);
exports.Mission = Mission;
